package queue

import (
	"context"
	"time"
	"tydata-server/app/user/cmd/api/internal/svc"

	"github.com/hibiken/asynq"
	"github.com/zeromicro/go-zero/core/logx"
)

// TASKTIME 定义为每天凌晨3点执行
const TASKTIME = "0 3 * * *"

type CleanQueryDataHandler struct {
	svcCtx *svc.ServiceContext
}

func NewCleanQueryDataHandler(svcCtx *svc.ServiceContext) *CleanQueryDataHandler {
	return &CleanQueryDataHandler{
		svcCtx: svcCtx,
	}
}

func (l *CleanQueryDataHandler) ProcessTask(ctx context.Context, t *asynq.Task) error {
	now := time.Now().Format("2006-01-02 15:04:05")
	logx.Infof("%s - 开始执行查询数据清理任务", now)

	// 计算3天前的时间
	threeDaysAgo := time.Now().AddDate(0, 0, -30)

	// 调用QueryModel删除3天前的数据
	result, err := l.svcCtx.QueryModel.DeleteBefore(ctx, threeDaysAgo)
	if err != nil {
		logx.Errorf("%s - 清理查询数据失败: %v", time.Now().Format("2006-01-02 15:04:05"), err)
		return err
	}

	logx.Infof("%s - 查询数据清理完成，共删除 %d 条记录", time.Now().Format("2006-01-02 15:04:05"), result)
	return nil
}
